# -*- coding: utf-8 -*-
#    phasecalculator calculates FGIPs, solvent similarity and VLE with SSIMPLE.
#    Copyright (C) 2019  Mark D. Driver
#
#    phasecalculator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script for running phase calculator calculations.

@author: Mark
"""

import logging
import pathlib
import phasecalculator.runners.phasexmlcreatorrunner as phasexmlrun
import phasecalculator.runners.fgipanalysisrunner as fgiprun
import phasecalculator.runners.similarityanalysisrunner as simrun
import phasecalculator.runners.vleanalysisrunner as vlerun


logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


def run_all_analysis(system_info, **kwargs):
    
    # TODO Logging output of analysis
    phase_filename, solvent_list = create_phase_and_solvent_xml_files(system_info, **kwargs)
    if system_info.calc_vle():
        run_vle_analysis(system_info, phase_filename, **kwargs)
    if system_info.calc_fgip():
        run_fgip_analysis(system_info, solvent_list, **kwargs)
    if system_info.calc_similarity():
        run_similarity_analysis(system_info, solvent_list, **kwargs)
    
def run_fgip_analysis(system_info, solvent_filename_dict_list, **kwargs):
    """
    

    Parameters
    ----------
    system_info : TYPE
        DESCRIPTION.
    solvent_filename_dict_list : TYPE
        DESCRIPTION.
    **kwargs : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """
    # TODO: for each solvent file/temp combo in dict
    jar_path = system_info.runtime_information.phasetransfer_jar
    combined_results = []
    combined_latex_blocks = ""
    for solvent_filename_dict in solvent_filename_dict_list:
        solvent_filename = solvent_filename_dict["solvent_filename"]
        energy_xml_filename = create_binding_energy_output_filename(solvent_filename)
        directory_base = system_info.runtime_information.output_dir
        temperature = solvent_filename_dict["temperature_value"]
        temperature_unit = solvent_filename_dict["temperature_units"]
        results, latex_blocks = fgiprun.calc_energies_and_fgips(jar_path, energy_xml_filename,
                                    solvent_filename, directory_base,
                                    temperature=temperature,
                                    temperature_unit=temperature_unit,
                                    **kwargs)
        combined_results.extend(results)
        combined_latex_blocks += latex_blocks
    return combined_results, combined_latex_blocks
def run_similarity_analysis(system_info, solvent_filename_dict, **kwargs):
    """

    Parameters
    ----------
    system_info : TYPE
        DESCRIPTION.
    solvent_filename_dict : TYPE
        DESCRIPTION.
    **kwargs : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """
    solvent_filename_dict_list = []
    jar_path = system_info.runtime_information.phasetransfer_jar
    for solv_dict in solvent_filename_dict:
        solvent_filename = solv_dict["solvent_filename"]
        energy_filename = create_free_energy_output_filename(solvent_filename)
        temperature = solv_dict["temperature_value"]
        temperature_unit = solv_dict["temperature_units"]
        solv_dict["energy_filename"] = energy_filename
        solvent_filename_dict_list.append(solv_dict)
        simrun.run_free_energy_calculation(jar_path, energy_filename,
                                           solvent_filename,
                                           temperature=temperature,
                                           temperature_unit=temperature_unit,
                                           **kwargs)
    output_dir = system_info.runtime_information.output_dir
    output_type = system_info.output_information.similarity_output_type
    out_res, poly_filename_list = simrun.extract_all_solvents_and_generate_polynomials(solvent_filename_dict_list,
                                                                                       output_dir)
    simrun.run_similarity_analysis(poly_filename_list, output_type, output_dir)
    #
    
def run_vle_analysis(system_info, phase_filename, **kwargs):
    """

    Parameters
    ----------
    system_info : TYPE
        DESCRIPTION.
    phase_filename : TYPE
        DESCRIPTION.
    **kwargs : TYPE
        DESCRIPTION.

    Returns
    -------
    TYPE
        DESCRIPTION.

    """
    jar_path = system_info.runtime_information.phasetransfer_jar
    output_filename = create_phase_output_filename(phase_filename)
    return vlerun.run_vle_calculation(jar_path, phase_filename, output_filename, **kwargs)

def create_output_dir(system_info):
    """Makes output directory for system calculation. 

    Parameters
    ----------
    system_info : System
        Calculation System object.

    Returns
    -------
    None.

    """
    output_path = pathlib.Path(system_info.runtime_information.output_dir)
    output_path.mkdir(parents=True,exist_ok=True)
    return output_path.as_posix()

def create_free_energy_output_filename(solvent_filename):
    """

    Parameters
    ----------
    solvent_filename : TYPE
        DESCRIPTION.

    Returns
    -------
    TYPE
        DESCRIPTION.

    """
    return solvent_filename.replace(".xml","free.xml")
def create_binding_energy_output_filename(solvent_filename):
    """

    Parameters
    ----------
    solvent_filename : TYPE
        DESCRIPTION.

    Returns
    -------
    TYPE
        DESCRIPTION.

    """
    return solvent_filename.replace(".xml", "binding.xml")
def create_phase_output_filename(phase_filename):
    """

    Parameters
    ----------
    phase_filename : TYPE
        DESCRIPTION.

    Returns
    -------
    TYPE
        DESCRIPTION.

    """
    return phase_filename.replace(".xml", "calculated.xml")

def create_phase_and_solvent_xml_files(system_info, **kwargs):
    """

    Parameters
    ----------
    system_info : TYPE
        DESCRIPTION.
    filestem : TYPE, optional
        DESCRIPTION. The default is "system".
    **kwargs : TYPE
        DESCRIPTION.

    Returns
    -------
    phase_file : TYPE
        DESCRIPTION.
    solvent_list : TYPE
        DESCRIPTION.

    """
    return phasexmlrun.create_phase_and_solvent_files(system_info, **kwargs)